"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.SshSessionClosedEventArgs = void 0;
const transportMessages_1 = require("../messages/transportMessages");
class SshSessionClosedEventArgs {
    constructor(reason, message, error) {
        this.reason = reason;
        this.message = message;
        this.error = error;
        if (!message)
            throw new TypeError('A disconnect message is required.');
    }
    toString() {
        return `${transportMessages_1.SshDisconnectReason[this.reason]}: ${this.message}`;
    }
}
exports.SshSessionClosedEventArgs = SshSessionClosedEventArgs;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@microsoft-dev-tunnels-ssh-events-sshSessionClosedEventArgs.js.map