export const defaultPerf = (typeof performance === 'object' &&
    performance &&
    typeof performance.now === 'function') ?
    /* c8 ignore start - this gets covered, but c8 gets confused */
    performance
    : /* c8 ignore stop */ Date;//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@xterm-addon-ligatures-node_modules-lru-cache-dist-esm-perf.js.map