"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tracing = exports.metrics = void 0;
const dummy = { hasSubscribers: false };
exports.metrics = dummy;
exports.tracing = dummy;//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@xterm-addon-ligatures-node_modules-lru-cache-dist-commonjs-diagnostics-channel-cjs.cjs.map