"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.SshKeepAliveEventArgs = void 0;
/**
 * Event args raised when an SSH keep-alive succeeds or fails.
 */
class SshKeepAliveEventArgs {
    constructor(count) {
        this.count = count;
    }
}
exports.SshKeepAliveEventArgs = SshKeepAliveEventArgs;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@microsoft-dev-tunnels-connections-sshKeepAliveEventArgs.js.map