"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.SshReportProgressEventArgs = void 0;
class SshReportProgressEventArgs {
    constructor(progress, sessionNumber) {
        this.progress = progress;
        this.sessionNumber = sessionNumber;
    }
    toString() {
        return `Progress: ${this.progress}` +
            this.sessionNumber ? ` Session number: ${this.sessionNumber}` : '';
    }
}
exports.SshReportProgressEventArgs = SshReportProgressEventArgs;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@microsoft-dev-tunnels-ssh-events-sshReportProgressEventArgs.js.map