const dummy = { hasSubscribers: false };
export const metrics = dummy;
export const tracing = dummy;//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@xterm-addon-ligatures-node_modules-lru-cache-dist-esm-browser-diagnostics-channel-browser.js.map