"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.PortForwardingEventArgs = void 0;
/**
 * Event raised when a port is about to be forwarded to the tunnel client.
 */
class PortForwardingEventArgs {
    /**
     * Creates a new instance of PortForwardingEventArgs.
     */
    constructor(
    /**
     * Gets the port number that is being forwarded.
     */
    portNumber) {
        this.portNumber = portNumber;
        this.cancel = false;
    }
}
exports.PortForwardingEventArgs = PortForwardingEventArgs;
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@microsoft-dev-tunnels-connections-portForwardingEventArgs.js.map