"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.umask = void 0;
// separate file so I stop getting nagged in vim about deprecated API.
const umask = () => process.umask();
exports.umask = umask;//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-tar-dist-commonjs-process-umask.js.map