"use strict";
/**
 * Copyright (c) 2020, Microsoft Corporation (MIT License).
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWorkerPipeName = getWorkerPipeName;
function getWorkerPipeName(conoutPipeName) {
    return "".concat(conoutPipeName, "-worker");
}//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-node-pty-lib-shared-conout.js.map