"use strict";
//
//  Copyright (c) Microsoft Corporation. All rights reserved.
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.curves = void 0;
/**
 * List of EC curves supported by the SSH ECDSA algorithm.
 */
exports.curves = [
    {
        shortName: 'P-256',
        name: 'nistp256',
        oid: '1.2.840.10045.3.1.7',
        keySize: 256,
    },
    {
        shortName: 'P-384',
        name: 'nistp384',
        oid: '1.3.132.0.34',
        keySize: 384,
    },
    {
        shortName: 'P-521',
        name: 'nistp521',
        oid: '1.3.132.0.35',
        keySize: 521,
    },
];
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@microsoft-dev-tunnels-ssh-algorithms-ecdsaCurves.js.map